import { b as createTemplate, a as addListener, d as doc } from "../index-CKRNGLIi.js";
import { createCopyButton } from "../extensions/copyButton/index.js";
import { l as getPosition, a as addOverlay, v as voidlessLangs, o as voidTags } from "../index-BvZmi6ce.js";
import { t as testBracket } from "../bracket-BPYnIBjq.js";
let counter = 0;
let sp;
const template = /* @__PURE__ */ createTemplate(
  "<div class=pce-tooltip style=z-index:5;top:auto;display:flex><div></div><div class=pce-hover-tooltip style=flex-shrink:0>"
);
const getLanguageAt = (token) => {
  return /language-(\S*)/.exec(token.closest("[class*=language-").className)[1];
};
const addHoverDescriptions = (codeBlock, callback, options = {}) => {
  let current;
  const { above, maxHeight, maxWidth } = options;
  const container = template();
  const pre = codeBlock.container;
  const style = container.style;
  const [spacer, tooltip] = container.children;
  const wrapper = codeBlock.wrapper;
  const id = tooltip.id = "pce-hover-" + counter++;
  const show = (target) => {
    const types = target.className.slice(6).split(" ");
    const text = target.textContent;
    const content = callback(types, getLanguageAt(target), text, target);
    if (content) {
      let { left, right, top, bottom, height } = getPosition(codeBlock, target);
      let { clientHeight, clientWidth } = pre;
      let max = bottom > top ? bottom : top;
      tooltip.style.maxWidth = `min(${maxWidth ? maxWidth + "," : ""}${clientWidth}px - var(--padding-left) - 1em)`;
      tooltip.style.maxHeight = `min(${maxHeight ? maxHeight + "," : ""}${max}px, ${clientHeight * 0.6}px - 2em)`;
      spacer.style.width = (pre.matches(".pce-rtl") ? right : left) + "px";
      tooltip.textContent = "";
      tooltip.append(...content);
      container.parentNode || addOverlay(codeBlock, container);
      let placeAbove = !above == top > bottom && (above ? top : bottom) < container.clientHeight ? !above : above;
      style[placeAbove ? "bottom" : "top"] = height + (placeAbove ? bottom : top) + "px";
      style[placeAbove ? "top" : "bottom"] = "auto";
      current?.removeAttribute("aria-describedby");
      target.setAttribute("aria-describedby", id);
      current = target;
    } else hide();
  };
  const hide = () => {
    current?.removeAttribute("aria-describedby");
    container.remove();
  };
  addListener(tooltip, "pointerleave", hide);
  addListener(wrapper, "pointerover", (e) => {
    const target = e.target;
    if (!tooltip.contains(target)) {
      if (target.matches(".token") && (e.pointerType != "mouse" || !e.buttons) && !target.childElementCount) {
        show(target);
      } else hide();
    }
  });
};
const highlightBracketPairsOnHover = (codeBlock, pairs = "()[]{}") => {
  highlightPairsOnHover(codeBlock, "active-bracket", "punctuation", (token, stack, map) => {
    const text = token.textContent;
    const last = text.length - 1;
    const bracketType = testBracket(text, pairs, last);
    if (bracketType) {
      if (bracketType % 2) stack[sp++] = [token, bracketType + 1];
      else {
        for (let i = sp; i; ) {
          let [el, type] = stack[--i];
          if (bracketType == type) {
            map.set(token, el);
            map.set(el, token);
            if (el.nextSibling == token) {
              el.textContent += token.textContent;
              token.textContent = "";
            }
            sp = i;
            i = 0;
          }
        }
      }
    }
  });
};
const highlightTagPairsOnHover = (codeBlock) => {
  const partialTags = [];
  const matchTag = (nameEl, isClosing, lastChild, stack, map) => {
    const tagName = nameEl.textContent;
    const notSelfClosing = !lastChild.textContent[1] && (voidlessLangs.has(getLanguageAt(nameEl)) || !voidTags.test(tagName));
    if (notSelfClosing) {
      if (isClosing) {
        for (let i = sp; i; ) {
          let entry = stack[--i];
          if (tagName == entry[1]) {
            map.set(nameEl, entry[0]);
            map.set(entry[0], nameEl);
            sp = i;
            i = 0;
          }
        }
      } else {
        stack[sp++] = [nameEl, tagName];
      }
    }
  };
  highlightPairsOnHover(codeBlock, "active-tagname", "tag", (token, stack, map) => {
    const children = token.children;
    const text = token.textContent;
    const lastChild = children[children.length - 1];
    const second = children[1];
    const hasClosingPunctuation = lastChild?.matches(".punctuation");
    if (second?.matches(".tag")) {
      if (hasClosingPunctuation) {
        matchTag(second, text[1] == "/", lastChild, stack, map);
      } else {
        partialTags.push([second, text[1] == "/"]);
      }
    } else if (hasClosingPunctuation && partialTags[0]) {
      matchTag(...partialTags.pop(), lastChild, stack, map);
    }
  });
};
const highlightPairsOnHover = (codeBlock, highlightClass, tokenName, forEachToken) => {
  let cache;
  const active = [[], []];
  const wrapper = codeBlock.wrapper;
  const toggleHighlight = (index, add) => active[index].forEach((el) => el.classList.toggle(highlightClass, !!add));
  const setCache = () => {
    cache = /* @__PURE__ */ new WeakMap();
    let tokens = wrapper.getElementsByClassName(tokenName);
    let i = sp = 0;
    let stack = [];
    let token;
    while (token = tokens[i++]) {
      forEachToken(token, stack, cache);
    }
  };
  const handler = (e) => {
    const target = e.target.closest("." + tokenName);
    const index = e.type == "click" ? 0 : 1;
    if (active[0].includes(target)) return;
    if (index && (e.pointerType != "mouse" || e.buttons)) return;
    if (!cache) setCache();
    toggleHighlight(index);
    active[index] = [];
    const other = cache.get(target);
    if (other) {
      active[1] = [];
      active[index] = [target, other];
      toggleHighlight(index, true);
    }
  };
  addListener(wrapper, "click", handler);
  addListener(wrapper, "pointerover", handler);
  addListener(wrapper, "pointerleave", () => {
    toggleHighlight(1);
    active[1] = [];
  });
};
const forEachCodeBlock = (root, callback) => {
  let els = root.getElementsByClassName("prism-code-editor");
  let i = 0;
  let result = [];
  while (i < els.length) {
    const element = els[i++];
    const json = element.dataset.props;
    if (!json) continue;
    const wrapper = element.firstChild;
    const lines = wrapper.children;
    const code = wrapper.textContent.slice(0, -1);
    const codeBlock = {
      container: element,
      wrapper,
      lines,
      code,
      language: /language-(\S*)/.exec(element.className)[1]
    };
    element.removeAttribute("data-props");
    callback(codeBlock, JSON.parse(json));
    result.push(codeBlock);
  }
  return result;
};
const omitLines = (selector) => (codeBlock) => {
  let result = "";
  let lines = codeBlock.lines;
  let i = 0;
  let line;
  while (line = lines[++i]) {
    if (!line.matches(selector)) result += line.textContent;
  }
  return result.slice(0, -1);
};
const addCopyButton = (codeBlock, getCode) => {
  const container = createCopyButton();
  const btn = container.firstChild;
  addListener(btn, "click", () => {
    btn.setAttribute("aria-label", "Copied!");
    if (!navigator.clipboard?.writeText(getCode ? getCode(codeBlock) : codeBlock.code)) {
      const selection = getSelection();
      const range = new Range();
      selection.removeAllRanges();
      selection.addRange(range);
      range.setStartAfter(codeBlock.lines[0]);
      range.setEndAfter(codeBlock.wrapper);
      doc.execCommand("copy");
      range.collapse();
    }
  });
  addListener(btn, "pointerenter", () => btn.setAttribute("aria-label", "Copy"));
  addOverlay(codeBlock, container);
};
export {
  addCopyButton,
  addHoverDescriptions,
  forEachCodeBlock,
  highlightBracketPairsOnHover,
  highlightTagPairsOnHover,
  omitLines
};
//# sourceMappingURL=code-block.js.map
